/***********************************************************************
**
** Module:		desktopservice.cpp
**
** Description:	This module implements our desktop service
**
** Author:		Geoffrey J. Elliott (GeoffE)
**
** Created:		02/26/99
**
** Modified:	
**
**				Copyright (C) 1999 Geoffrey J. Elliott
**				All Rights Reserved
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
** 
** The author can be contacted via Email at geo@cybersnot.com
**
***********************************************************************/

#include <windows.h>
#include "../../common/API.h"
#include "desktopservice.h"
#include "launchService.h"
#include "notificationService.h"
#include "pluginService.h"

//
// Debugging support:
//
#define _dbgModule	"desktopservice.cpp"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
CDesktopService::CDesktopService()
{
	dbgTraceEnter("CDesktopService::CDesktopService");
	dbgTraceLeave();
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
CDesktopService::~CDesktopService()
{
	dbgTraceEnter("CDesktopService::~CDesktopService");

	UnregisterHotKey(m_hwnd, GlobalFindAtom("geODesktop_Exit"));
	UnregisterHotKey(m_hwnd, GlobalFindAtom("geODesktop_NewHost"));
	UnregisterHotKey(m_hwnd, GlobalFindAtom("geODesktop_Run"));
	UnregisterHotKey(m_hwnd, GlobalFindAtom("geODesktop_Shutdown"));
	UnregisterHotKey(m_hwnd, GlobalFindAtom("geODesktop_Logoff"));

	GlobalDeleteAtom(GlobalFindAtom("geODesktop_Exit"));
	GlobalDeleteAtom(GlobalFindAtom("geODesktop_NewHost"));
	GlobalDeleteAtom(GlobalFindAtom("geODesktop_Run"));
	GlobalDeleteAtom(GlobalFindAtom("geODesktop_Shutdown"));
	GlobalDeleteAtom(GlobalFindAtom("geODesktop_Logoff"));

	UnregisterClass("DesktopBackgroundClass", m_hInst);

	dbgTraceLeave();
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Startup
//
// Don't mess with the desktop, baybee.
//
HRESULT	CDesktopService::Startup(HINSTANCE hInst, HWND hwndParent)
{
	dbgTraceEnter("CDesktopService::Startup");

	WNDCLASS	wc;

	//
	// Remember our module settings:
	//
	m_hInst = hInst;
	m_hwndParent = hwndParent;

   
   ////////////////////////////////////
   // Added by SamIam
   // Sets the size of the desktop 
   //

   setMinMax();

   //
   //End Added by SamIam
   ////////////////////////////////////


	//
	// Get need to know some coordinates:
	//
	m_screenWidth = GetSystemMetrics(SM_CXSCREEN);
	m_screenHeight = GetSystemMetrics(SM_CYSCREEN);

	//
	// Let's create our desktop's window class:
	//
	memset(&wc,0,sizeof(wc));
	wc.lpfnWndProc = _WndProc;			// our window procedure
	wc.hInstance = hInst;					// hInstance of DLL
	wc.lpszClassName = "DesktopBackgroundClass";		// our window class name
	wc.style = CS_DBLCLKS;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	if (FAILED(_RegisterClass(&wc))) {
		return E_OUTOFMEMORY;
	}

	//
	// Okey smokes, let's create our window.
	//
	m_hwnd = CreateWindowEx(WS_EX_TOOLWINDOW,						// exstyles 
							"DesktopBackgroundClass",				// our window class name
							"geoShell :: Desktop Window",			// use description for a window title
							WS_POPUP|WS_CLIPSIBLINGS|WS_CLIPCHILDREN,				
							0, 0,									// position 
							m_screenWidth,m_screenHeight,			// width & height of window
							hwndParent,								// parent window 
							NULL,									// no menu
							m_hInst,								// hInstance of DLL
							this);									// no window creation data
    SetWindowPos(m_hwnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);
	ShowWindow(m_hwnd, SW_SHOWNORMAL);
	
	//
	// Add our default hotkeys...
	//
	RegisterHotKey(m_hwnd, GlobalAddAtom("geODesktop_Exit"), MOD_WIN, 'X');
	RegisterHotKey(m_hwnd, GlobalAddAtom("geODesktop_NewHost"), MOD_WIN, 'H');
	RegisterHotKey(m_hwnd, GlobalAddAtom("geODesktop_Run"), MOD_WIN, 'R');
	RegisterHotKey(m_hwnd, GlobalAddAtom("geODesktop_Shutdown"), MOD_WIN, 'Q');
	RegisterHotKey(m_hwnd, GlobalAddAtom("geODesktop_Logoff"), MOD_WIN, 'L');

	//
	// Make ourselves the official shell window.
	//
	// BUGBUG: This seems to break Explorer.exe, at least on NT4.
	//
    FARPROC (__stdcall *SetShellWindow)(HWND) = NULL;
    SetShellWindow = (FARPROC (__stdcall *)(HWND))GetProcAddress(GetModuleHandle("USER32.DLL"), "SetShellWindow");
	//SetShellWindow(m_hwnd);

	dbgTraceLeave();
	return S_OK;
}


HRESULT	CDesktopService::Shutdown(VOID)
{
	dbgTraceEnter("CDesktopService::Shutdown");

   ////////////////////////////////////
   // Added by SamIam
   // Sets the size of the desktop 
   //

   resetMinMax();

   //
   //End Added by SamIam
   ////////////////////////////////////



	delete this;

	dbgTraceLeave();
	return S_OK;
}

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
HWND CDesktopService::Hwnd(VOID) {
	dbgTraceEnter("CDesktopService::hwnd");

	dbgTraceLeave();
	return m_hwnd;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT CDesktopService::wndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_HOTKEY:
		{
			UINT fuModifiers = (UINT)LOWORD(lParam);
			UINT uVirtKey = (UINT)HIWORD(lParam);
			if (fuModifiers == MOD_WIN) {
				switch(uVirtKey) {
				case 'X':
					//
					// Quit the shell...
					//
					PostQuitMessage(0);
					break;

				case 'H':
					//
					// Create an empty geoBar:
					//
					g_pluginService->OpenHost(NULL);
					break;

				case 'R':
					//
					// Do the run dialog:
					//
					{
						HRESULT (__stdcall *SHRunDlg)(HWND hwnd, INT x, INT y, CHAR *sz1, CHAR *sz2, INT i) = NULL;
						SHRunDlg = (HRESULT (__stdcall *)(HWND hwnd, INT, INT, CHAR *, CHAR *, INT))GetProcAddress(GetModuleHandle("SHELL32.DLL"), (CHAR *)((LONG)0x3D));
						if (SHRunDlg != NULL) {
							if SUCCEEDED(SHRunDlg(hwnd, NULL, NULL, NULL, NULL, FALSE)) {
								return 1;
							}
						}
					}
					break;

				case 'Q':
					//
					// Do the shutdown dialog:
					//
					{
						FARPROC (__stdcall *MSWinShutdown)(HWND) = NULL;
						MSWinShutdown = (FARPROC (__stdcall *)(HWND))GetProcAddress(GetModuleHandle("SHELL32.DLL"), (char*)((long)0x3C));
						MSWinShutdown(m_hwnd);
					}
					break;

				case 'L':
					//
					// Log the user off:
					//
					if (MessageBox(hwnd, "Are you sure you want to log off?", "Log Off Windows", MB_YESNO | MB_ICONQUESTION | MB_SYSTEMMODAL | MB_SETFOREGROUND | MB_TOPMOST) == IDYES) {
						PostQuitMessage(0);
						ExitWindowsEx(EWX_LOGOFF, 0);
					}
				}			
			} else {
				g_launchService->NotifyViaHotkey(wParam, WM_HOTKEY);
			}
		}
		return 0;
		break;

	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
		break;

    case WM_DISPLAYCHANGE:
		return _wmDisplayChange(LOWORD(lParam), HIWORD(lParam));
		break;

	case WM_DEVICECHANGE:
		/*
		PDEV_BROADCAST_HDR pdbch;
		PDEV_BROADCAST_VOLUME pdbcv;
		if (wParam != DBT_DEVICEARRIVAL) return TRUE;
		pdbch = (PDEV_BROADCAST_HDR) lParam;
		switch (pdbch->dbch_devicetype)
			{
			case DBT_DEVTYP_VOLUME:
				pdbcv = (PDEV_BROADCAST_VOLUME) pdbch;
				if (pdbcv->dbcv_flags == DBTF_MEDIA) 
					{
					int i;
					for (i =0; i< 26;i++)
						{
						if (pdbcv->dbcv_unitmask & (1 << i))
                                                            autorun(i); // Call autorun on a specific drive
						}
					}
				return TRUE;
			default:
				return TRUE;
			}
		}*/

		//
		// Grant the request (return TRUE).  If we don't
		// want the request to process, we can return FALSE.
		//
		return TRUE;
		break;

    case WM_MOUSEACTIVATE:
		return _wmMouseActivate();
		break;

    case WM_ACTIVATE:
		return _wmActivate();
		break;

	case WM_ERASEBKGND:
		return _wmEraseBackground((HDC)wParam);
		break;

	case WM_PAINT:
		return _wmPaint();
		break;

	case WM_WINDOWPOSCHANGING:
		return _wmWindowPosChanging((WINDOWPOS *)lParam);
		break;

    case WM_NCPAINT:
		return _wmNCPaint((HRGN)wParam);
        break;

    case WM_RBUTTONUP:
		return _wmRButtonUp(LOWORD(lParam), HIWORD(lParam));
        break;

	default:
		return DefWindowProc(hwnd,uMsg,wParam,lParam);
		break;
	}
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT	CDesktopService::_wmDisplayChange(UINT screenWidth, UINT screenHeight)
{
	dbgTraceEnter("CDesktopService::_wmDisplayChange");

	//
	// Oh gawsh... the display size has changed...
	//
	m_screenWidth = screenWidth;
	m_screenHeight = screenHeight;
	dbgTraceArg("Screen width has changed to '%d'", m_screenWidth);
	dbgTraceArg("Screen height has changed to '%d'", m_screenHeight);

	//
	// Ok, resize the desktop to the new coords.
	//
	SetWindowPos(m_hwnd, HWND_BOTTOM, 0, 0, m_screenWidth, m_screenHeight, SWP_NOMOVE);

	dbgTraceLeave();
	return FALSE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT CDesktopService::_wmMouseActivate(VOID)
{
	dbgTraceEnter("CDesktopService::_wmMouseActivate");

	dbgTraceLeave();
	return MA_ACTIVATE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT	CDesktopService::_wmActivate(VOID)
{
	dbgTraceEnter("CDesktopService::_wmActivate");

	dbgTraceLeave();
	return FALSE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT	CDesktopService::_wmEndSession(VOID)
{
	dbgTraceEnter("CDesktopService::_wmEndSession");

	//
	// Close down...
	//
	PostQuitMessage(0);

	//
	// We *MUST* return TRUE here:
	//
	dbgTraceLeave();
	return TRUE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT	CDesktopService::_wmNotifyParent(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	dbgTraceEnter("CDesktopService::_wmNotifyParent");

	//
	// Make our parent aware of this message...
	//
	dbgTraceLeave();
	return ::PostMessage(m_hwndParent, uMsg, wParam, lParam);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT	CDesktopService::_wmEraseBackground(HDC /* hdc */)
{
	dbgTraceEnter("CDesktopService::_wmEraseBackground");
	dbgTraceLeave();
	return FALSE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT	CDesktopService::_wmPaint(VOID)
{
	dbgTraceEnter("CDesktopService::_wmPaint");

	PAINTSTRUCT ps;
	HDC			hdc;

	hdc = BeginPaint(m_hwnd,&ps);
	PaintDesktop(hdc);
  	EndPaint(m_hwnd,&ps);
	
	dbgTraceLeave();
	return FALSE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT CDesktopService::_wmWindowPosChanging(WINDOWPOS *wpPosition)
{
	dbgTraceEnter("CDesktopService::_wmWindowPosChanging");

	//
	// Hey, you can't change my position!  don't even
	// try it!
	//
	wpPosition->hwndInsertAfter = HWND_BOTTOM;
	wpPosition->flags |= SWP_NOMOVE | SWP_NOSIZE;

	dbgTraceLeave();
	return FALSE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//
//
//
//
LRESULT CDesktopService::_wmNCPaint(HRGN hrgn)
{
	dbgTraceEnter("CDesktopService::_wmNCPaint");

	//
	// Hmm, trying to move us with a non-client paint
	// are you?  Well, NO!
	//
    if (hrgn == NULL) {
        SetWindowPos(m_hwnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);
	}

	dbgTraceLeave();
	return FALSE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// _wmRButtonUp
//
// Hey, they want a desktop menu -- let's give it to them!
//
const	DWORD	MI_DesktopProperties = 1;
const	DWORD	MI_NewGeoBar = 2;
const	DWORD	MI_LogOff = 3;
const	DWORD	MI_Shutdown = 4;
const	DWORD	MI_Refresh = 5;
const	DWORD	MI_About = 6;
const	DWORD	MI_PluginFirst = 100;
LRESULT CDesktopService::_wmRButtonUp(INT x, INT y)
{
	dbgTraceEnter("CDesktopService::_wmRButtonUp");

	HMENU	hMenu;
	HMENU	hSubMenu;
	DWORD	dwAlign;
	UINT	i;
	
	hMenu = CreatePopupMenu();
	if (hMenu != NULL) {
		AppendMenu(hMenu, MF_STRING, MI_About, "&About geOShell");
		AppendMenu(hMenu, MF_SEPARATOR | MF_DISABLED, 0, NULL);

		//
		// Refresh the desktop
		//
		AppendMenu(hMenu, MF_STRING, MI_Refresh, "Re&fresh");
		AppendMenu(hMenu, MF_SEPARATOR | MF_DISABLED, 0, NULL);

		//
		// New Menu
		//
		hSubMenu = CreatePopupMenu();
		if (hSubMenu != NULL) {
			AppendMenu(hSubMenu, MF_STRING, MI_NewGeoBar, "&geOBar");
			AppendMenu(hSubMenu, MF_SEPARATOR | MF_DISABLED, 0, NULL);

			//
			// BUGBUG: Should be in the plugin manager or something?
			//
			HRESULT hr;
			HKEY	hkPlugins;
			hr = HRESULT_FROM_WIN32(RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\geO\\geOShell\\Plugins", 0, KEY_READ, &hkPlugins));
			if (SUCCEEDED(hr)) {
				DWORD dwIndex = 0;
				DWORD dwSize;
				CHAR szName[MAX_PATH+1];
				dwSize = MAX_PATH;
				hr = HRESULT_FROM_WIN32(RegEnumValue(hkPlugins, dwIndex, szName, &dwSize, NULL, NULL, NULL, NULL));
				while (SUCCEEDED(hr)) {
					AppendMenu(hSubMenu, MF_STRING, MI_PluginFirst + dwIndex, szName);
					dwIndex++;
					dwSize = MAX_PATH;
					hr = HRESULT_FROM_WIN32(RegEnumValue(hkPlugins, dwIndex, szName, &dwSize, NULL, NULL, NULL, NULL));
				}
				RegCloseKey(hkPlugins);
			}
	
			AppendMenu(hMenu, MF_STRING | MF_POPUP, (UINT)hSubMenu, "&New");
		}
		AppendMenu(hMenu, MF_SEPARATOR | MF_DISABLED, 0, NULL);

		//
		// Properties
		//
		AppendMenu(hMenu, MF_STRING, MI_DesktopProperties, "P&roperties");
		AppendMenu(hMenu, MF_SEPARATOR | MF_DISABLED, 0, NULL);

		//
		// Shutdown / Log Off
		//
		AppendMenu(hMenu, MF_STRING, MI_LogOff, "&Log Off");
		AppendMenu(hMenu, MF_STRING, MI_Shutdown, "Sh&ut Down");

		//
		// Okay, let's figure out how to align the menu:
		//
		dwAlign = 0;
		if (x > (GetSystemMetrics(SM_CXSCREEN) / 2)) {
			dwAlign |= TPM_RIGHTALIGN;
		} else {
			dwAlign |= TPM_LEFTALIGN;
		}
		if (y > (GetSystemMetrics(SM_CYSCREEN) / 2)) {
			dwAlign |= TPM_BOTTOMALIGN;
		} else {
			dwAlign |= TPM_TOPALIGN;
		}

		i = TrackPopupMenu(hMenu, dwAlign | TPM_RETURNCMD | TPM_LEFTBUTTON, x, y, 0, m_hwnd, NULL);
		if (i > 0) {
			switch(i) {
			case MI_DesktopProperties:
				{
					CHAR szDir[MAX_PATH+1];
					GetSystemDirectory(szDir, MAX_PATH);
					ShellExecute(m_hwnd, "open", "rundll32.exe", "shell32.dll,Control_RunDLL desk.cpl", szDir, SW_SHOWNORMAL);
				}
				break;

			case MI_NewGeoBar:
				SendMessage(WM_HOTKEY, NULL, MAKELONG(MOD_WIN, 'H'));
				break;

			case MI_LogOff:
				SendMessage(WM_HOTKEY, NULL, MAKELONG(MOD_WIN, 'L'));
				break;

			case MI_Shutdown:
				SendMessage(WM_HOTKEY, NULL, MAKELONG(MOD_WIN, 'Q'));
				break;

			case MI_Refresh:
				InvalidateRect(m_hwnd, NULL, TRUE);
				break;

			case MI_About:
				MessageBox(m_hwnd, "geOShell\nA replacement shell for Windows.\n\nCopyright (C) 1999 Geoffrey J. Elliott\nAll Rights Reserved.", "About geOShell...", MB_OK | MB_ICONINFORMATION);
				break;

			default:
				if (i >= MI_PluginFirst) {
					CHAR szMenu[MAX_PATH+1];
					GetMenuString(hMenu, i, szMenu, MAX_PATH, MF_BYCOMMAND);			
					g_pluginService->LoadPlugin(NULL, szMenu, NULL);
				}
			}
		}
	}
		
	dbgTraceLeave();
	return FALSE;
}


LRESULT	CDesktopService::SendMessage(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return ::SendMessage(m_hwnd, uMsg, wParam, lParam);
}

LRESULT	CDesktopService::PostMessage(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return ::PostMessage(m_hwnd, uMsg, wParam, lParam);
}

/*
//////////////////////////////////////////////////////////////////
// Added by SamIam
// Sets the size of the desktop 
//
void CDesktopService::setMinMax()
{
	HKEY	hk;
	HRESULT	hr;

	RECT r;
   DWORD dwValue;
   UINT sdaLeft = 0, sdaRight = 0, sdaBottom = 0, sdaTop = 0;

	hr = HRESULT_FROM_WIN32(RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\geO\\geOShell\\Services\\Desktop", NULL, KEY_READ, &hk));
	if (SUCCEEDED(hr)) {
      dwValue = _RegGetValueDW(hk, "SetDesktopArea", 0);
      if (dwValue == 1){
		   sdaLeft = (UINT)_RegGetValueDW(hk, "SDALeft", 0);
         sdaRight = (UINT)_RegGetValueDW(hk, "SDAWidth", 0);
         sdaBottom = (UINT)_RegGetValueDW(hk, "SDAHeight", 0);
         sdaTop = (UINT)_RegGetValueDW(hk, "SDATop", 0);
	
   	   SystemParametersInfo(SPI_GETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
         SetDesktopArea(r.left, r.top, r.right, r.bottom);

         r.left = sdaLeft;
	      r.top = sdaTop;
	      r.right = GetSystemMetrics(SM_CXSCREEN);
	      r.bottom = GetSystemMetrics(SM_CYSCREEN);
	      r.right = sdaRight<0 ? r.right+sdaRight:sdaRight;
	      r.bottom = sdaBottom<0 ? r.bottom+sdaBottom:sdaBottom;
	   
         SetDesktopArea(r.left, r.top, r.right, r.bottom);
      }
	   RegCloseKey(hk);
	}
}


void CDesktopService::resetMinMax()
{
	RECT r;	

	r.right = GetSystemMetrics(SM_CXSCREEN);
	r.bottom = GetSystemMetrics(SM_CYSCREEN);
   r.top = 0;
   r.left = 0;

   SetDesktopArea(r.left, r.top, r.right, r.bottom);
}
//
// End Added by SamIam
///////////////////////////////////////////////////////////////////////
*/

//////////////////////////////////////////////////////////////////
// Added by SamIam
// Sets the size of the desktop 
//
void CDesktopService::setMinMax()
{
	HKEY	hk;
	HRESULT	hr;

	RECT r;
   DWORD dwValue;
   UINT sdaLeft = 0, sdaRight = 0, sdaBottom = 0, sdaTop = 0;

	hr = HRESULT_FROM_WIN32(RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\geO\\geOShell\\Services\\Desktop", NULL, KEY_READ, &hk));
	if (SUCCEEDED(hr)) {
      dwValue = _RegGetValueDW(hk, "SetDesktopArea", 0);
      if (dwValue == 1){
         /*MessageBox(NULL,"Setting Desktop area","bmfmf",MB_OK);*/
		   sdaLeft = (UINT)_RegGetValueDW(hk, "SDALeft", 0);
         sdaRight = (UINT)_RegGetValueDW(hk, "SDAWidth", 0);
         sdaBottom = (UINT)_RegGetValueDW(hk, "SDAHeight", 0);
         sdaTop = (UINT)_RegGetValueDW(hk, "SDATop", 0);
	
   	   SystemParametersInfo(SPI_GETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
         SetDesktopArea(r.left, r.top, r.right, r.bottom);

         r.left = sdaLeft;
	      r.top = sdaTop;
	      r.right = GetSystemMetrics(SM_CXSCREEN);
	      r.bottom = GetSystemMetrics(SM_CYSCREEN);
	      r.right = sdaRight<0 ? r.right+sdaRight:sdaRight;
	      r.bottom = sdaBottom<0 ? r.bottom+sdaBottom:sdaBottom;
	   
         SetDesktopArea(r.left, r.top, r.right, r.bottom);
      }
	   RegCloseKey(hk);
	}
}
void CDesktopService::SetDesktopArea(int left, int top, int right, int bottom)
{
	RECT r;

	r.left = left;
	r.top = top;
	r.right = right;
	r.bottom = bottom;

   /*char szValue[MAX_PATH];   
   itoa(r.top,szValue,10);
   MessageBox(NULL,szValue,"Top",MB_OK);
   itoa(r.left,szValue,10);
   MessageBox(NULL,szValue,"Left",MB_OK);
   itoa(r.right,szValue,10);
   MessageBox(NULL,szValue,"Right",MB_OK);
   itoa(r.bottom,szValue,10);
   MessageBox(NULL,szValue,"Bottom",MB_OK);*/

	SystemParametersInfo(SPI_SETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
	SystemParametersInfo(SPI_GETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
}

void CDesktopService::resetMinMax()
{
	RECT r;	

   SystemParametersInfo(SPI_GETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
   SetDesktopArea(r.left, r.top, r.right, r.bottom);

	r.right = GetSystemMetrics(SM_CXSCREEN);
	r.bottom = GetSystemMetrics(SM_CYSCREEN);
   r.top = 0;
   r.left = 0;

   SetDesktopArea(r.left, r.top, r.right, r.bottom);
}
//
// End Added by SamIam
///////////////////////////////////////////////////////////////////////
